/*=================================================================*/
/* From the Combinatorial Object Server:                           */
/* Generates the k-combinations of [n] by transpositions           */
/* No input error checking.  Assumes 0 <= k <= n <= MAX.           */
/* Outputs both the bitstring and the transposition (x,y) (meaning */
/* that x leaves the subset and y enters).                         */
/* Algorithm is CAT (Constant Amortized Time).                     */
/* Original version of this program by Frank Ruskey (1995) can be  */
/* found at http://sue.uvic.ca/~cos/inf/comb/CombinationsInfo.html */
/*=================================================================*/

#include <stdio.h>

void NEG( int, int);
void GEN( int, int); 
void PrintIt();

    int n, k;
    int a;

void main(void)
{
    printf( "Enter n,k: ");
    scanf( "%d,%d", &n, &k );
    
    a = (1 << k) - 1;
    a <<= (n-k);

    PrintIt(); 
    GEN( n, k);
}


void PrintIt()
{
    int i, d;
    for (i = 1, d=1; i <= n; i++, d <<= 1) 
        printf( "%d ", (a&d) ? 1 : 0);
    printf( "\n");
}


void swap ( int x, int y)
{
    a |= (1<<(n-x)); a &= ~(1<<(n-y));
    PrintIt(); 
}


void GEN( int n, int k)
{
    if (k > 0 && k < n)
        {
        GEN( n-1, k);

        if (k == 1)
            swap( n, n-1 );
        else
            swap( n, k-1 );

        NEG( n-1, k-1);
        }
}


void NEG( int n, int k)
{
    if (k > 0 && k < n)
        {
        GEN( n-1, k-1 );

        if (k == 1)
            swap( n-1, n);
        else
            swap( k-1, n);
        
        NEG( n-1, k);
        }
}
