#include <stdio.h>
#define MAXN 10

/***********************************************************
* FUNCTION NAME: showProduct
*
* DESCRIPTION:  shows a set crossproduct
*
* ARGUMENT LIST:
* Argument        Type   IO      Description
* ------------- -------- -- --------------------------------
* int           n       I   number ofelements to show
* int           prodNb  I   ordering number of product
* int printedLinesCount I   count of printed lines so far
* int           p[]     I   array of product elements
* RETURN VALUE: void
*
* CHANGES: -
*************************************************************/
void showProduct(const int n, const int printedLinesCount, const int prodNb, const int p[])
{
    int i;

    printf("\n%3d ", prodNb);
    for (i = 1; i <= n; i++) printf(" %2d", p[i]);
    if ( printedLinesCount % 24 == 0 ) getchar();
}
/***********************************************************
* FUNCTION NAME: crossProdNonRec
*
* DESCRIPTION:  generates the elements of the cross product
*               of n sets and prints them to stdout
*
* ARGUMENT LIST:
* Argument        Type   IO      Description
* ------------- -------- -- --------------------------------
* int           n         I number of sets;
* int printedLinesCount   I   count of printed lines so far
* int           nElem[]   I vector with number of elements per se
* RETURN VALUE: void
*
* CHANGES: -
*************************************************************/
void crossProdNonRec(const int n, int printedLinesCount, const int nElem[])
{
    int i, prodNb, p[MAXN];

    prodNb = 1;
    for (i = 1; i <= n; i++) p[i] = 1;
    showProduct(n, printedLinesCount++, prodNb, p);
    i = n;
    while ( i > 0 )
    {
        p[i]++;
        if (p[i] > nElem[i])
        {
            p[i] = 1;
            i--;
        }
        else
        {
            prodNb++;
            showProduct(n, printedLinesCount++, prodNb, p);
            i = n;
        }
    }
}
int main()
{
    int i, n, nElem[MAXN], printedLinesCount=0;
    printf("Number of sets [<%d]=", MAXN);
    scanf("%d", &n);
    printedLinesCount++;
    for (i = 1; i <= n; i++)
    {
        printf("Number of elements in set %d=", i);
        scanf("%d%*c", &nElem[i]);
        printedLinesCount++;
    }
    printf("The members of the cross product\nNo.   Elements");
    printedLinesCount += 2;
    crossProdNonRec(n, printedLinesCount, nElem);
    return 0;
}
